@@ -250,8 +250,11 @@ public class Version {
     public static final Version V_1_7_2 = new Version(V_1_7_2_ID, false, org.apache.lucene.util.Version.LUCENE_4_10_4);
     public static final int V_1_7_3_ID = 1070399;
     public static final Version V_1_7_3 = new Version(V_1_7_3_ID, false, org.apache.lucene.util.Version.LUCENE_4_10_4);
-
-    public static final Version CURRENT = V_1_7_3;
+    public static final int V_1_7_4_ID = 1070499;
+    public static final Version V_1_7_4 = new Version(V_1_7_4_ID, false, org.apache.lucene.util.Version.LUCENE_4_10_4);
+    public static final int V_1_7_5_ID = 1070599;
+    public static final Version V_1_7_5 = new Version(V_1_7_5_ID, false, org.apache.lucene.util.Version.LUCENE_4_10_4);
+    public static final Version CURRENT = V_1_7_5;
 
     static {
         assert CURRENT.luceneVersion.equals(Lucene.VERSION) : "Version must be upgraded to [" + Lucene.VERSION + "] is still set to [" + CURRENT.luceneVersion + "]";
@@ -263,6 +266,10 @@ public class Version {
 
     public static Version fromId(int id) {
         switch (id) {
+            case V_1_7_5_ID:
+                return V_1_7_5;
+            case V_1_7_4_ID:
+                return V_1_7_4;
             case V_1_7_3_ID:
                 return V_1_7_3;
             case V_1_7_2_ID:
